import pandas as pd
import numpy as np

def euclidean(point, data):
    return np.sqrt(np.sum((data - point)**2, axis=1))

def euclidean_v(x, y):
    """
    Euclidean distance is the square root of the sum of the squared differences of their coordinates
    """
    return np.sqrt(np.sum((x - y)**2))

def manhattan_v(x, y):
    """
    Manhattan distance is the sum of the absolute differences of their coordinates
    """
    return np.sum(np.abs(x - y))

def cosine_v(x, y):
    """
    Cosine distance is 1 - cosine similarity
    """
    return 1 - np.dot(x, y) / (np.linalg.norm(x) * np.linalg.norm(y))

## REGRESSION METRICS ##
def mean_absolute_percentage_error(actual, predicted):
    """
    Local Mean Absolute Percentage Error (LMAPE)
    """
    return np.mean(np.abs((actual - predicted) / actual))

def mean_absolute_error(actual, predicted):
    """
    Mean Absolute Error (MAE)
    """
    return np.mean(np.abs(predicted-actual.T).T)

def root_mean_squared_error(actual, predicted):
    """
    Local Root Mean Squared Error (LRMSE)
    """
    return np.sqrt(np.mean((predicted-actual.T).T)**2)

def mean_squared_error(actual, predicted):
    """
    Mean Squared Error (MSE)
    """
    return np.mean((predicted-actual.T).T)**2

def root_mean_squared_log_error(actual, predicted):
    """
    Local Root Mean Squared Log Error (LRMSLE)
    """
    return np.sqrt(np.mean(np.square(np.log(predicted+1) - np.log(actual+1))))


def metrics_table(actual, predicted, model_name):
    """
    Create a table with pivot with results of multiple models and metrics
    """
    metrics = pd.DataFrame(columns=['Model', 'MAPE', 'MAE', 'RMSE', 'RMSLE'],
                           data=[[model_name, mean_absolute_percentage_error(actual, predicted),
                                  mean_absolute_error(actual, predicted),
                                  root_mean_squared_error(actual, predicted),
                                  root_mean_squared_log_error(actual, predicted)]])

    return metrics #.to_markdown()




from .metrics import *

def w_inverse_LMAE(actual, predicted):
    """
    Inverse Local MAE
    """
    return 1/mean_absolute_error(actual, predicted)

def w_inverse_log_LMAE(actual, predicted):
    """
    Inverse Log Local MAE
    """
    return np.log(max(abs(actual-predicted))/mean_absolute_error(actual, predicted))

def get_k_nearest_neighbors(point, data, k, metric):
    """
    Get the k nearest neighbors of a point in a dataset
    """
    distances = metric(point, data)
    return distances.argsort()[:k]

def get_k_nearest_neighbors_weights(point, data, k, metric, weights):
    """
    Get the k nearest neighbors of a point in a dataset weighing the neighbors
    """
    distances = metric(point, data)
    return distances.argsort()[:k], weights(distances)

def predict_inverse_LMAE(point, data, k, metric):
    """
    Predict the target value of a point using the inverse LMAE
    """
    neighbors = get_k_nearest_neighbors(point, data, k, metric)
    return np.average(data[neighbors], axis=0, weights=w_inverse_LMAE)

def error_bias(data, k, metric):
    """
    Calculate the bias of the error
    """
    error_bias = []
    for i in range(len(data)):
        neighbors = get_k_nearest_neighbors(data[i], data, k, metric)
        error_bias.append(np.sum(data[neighbors] - data[i])/k)
    return error_bias


import numpy as np

def split_sets(df, train_size, val_size, test_size, target):
    """
    Split the data into train, validation and test sets with target and features
    """
    train, val, test = divide_sets(df, train_size, val_size, test_size)
    return train.drop(columns=[target]), \
           train[target], \
           val.drop(columns=[target]), \
           val[target], \
           test.drop(columns=[target]), \
           test[target]

def divide_sets(df, train_size, val_size, test_size):
    """
    Divide the data into train, validation and test sets
    """
    train_size = int(train_size * len(df))
    val_size = int(val_size * len(df))
    test_size = int(test_size * len(df))
    return df[:train_size], df[train_size:train_size+val_size], df[train_size+val_size:train_size+val_size+test_size]

def euclidean_v(x, y):
    return np.sqrt(np.sum((x - y)**2))


def manhattan_v(x, y):
    return np.sum(np.abs(x - y))


def cosine_v(x, y):
    return 1 - np.dot(x, y) / (np.linalg.norm(x) * np.linalg.norm(y))



import setuptools

with open("README.md", "r", encoding = "utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "ensemblem",
    version = "0.2.7",
    author = "Ivan Vigorito",
    author_email = "ivanvigorito1994@gmail.com",
    description = "Dynamic Weighted Ensemble - Local Fusion",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/IvanVigor/Dynamic-Weighted-Ensemble",
    project_urls = {
        "Bug Tracker": "https://github.com/IvanVigor/Local-Fusion-Dynamic-Weighted-Ensemble/issues",
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir = {"": "src"},
    packages = setuptools.find_packages(where="src"),
    python_requires = ">=3.8"
)